@@ -1,20 +1,16 @@
 """
 Additional implementation of :mod:`sklearndf.classification` loaded
-from sklearn 0.22 onwards
+from sklearn 0.23 onwards
 """
-import logging
 
-from sklearn.ensemble import StackingClassifier
-from sklearn.naive_bayes import CategoricalNB
+import logging
+from typing import List
 
 from pytools.api import AllTracker
 
-from ..wrapper import make_df_classifier
-from .wrapper._wrapper import StackingClassifierWrapperDF
-
 log = logging.getLogger(__name__)
 
-__all__ = ["CategoricalNBDF", "StackingClassifierDF"]
+__all__: List[str] = []
 
 __imported_estimators = {name for name in globals().keys() if name.endswith("DF")}
 
@@ -23,33 +19,19 @@ __imported_estimators = {name for name in globals().keys() if name.endswith("DF"
 # Ensure all symbols introduced below are included in __all__
 #
 
-__tracker = AllTracker(globals(), allow_imported_definitions=True)
+__tracker = AllTracker(globals())
 
 
 #
 # Class definitions
 #
 
-# todo: add other classification implementations for sklearn 0.22
 
-#
-# naive bayes
-#
+# todo: add classification implementations for sklearn 0.23
 
-CategoricalNBDF = make_df_classifier(CategoricalNB)
-
-StackingClassifierDF = make_df_classifier(
-    StackingClassifier, base_wrapper=StackingClassifierWrapperDF
-)
-
-
-#
-# validate __all__
-#
 
 __tracker.validate()
 
-
 #
 # validate that __all__ comprises all symbols ending in "DF", and no others
 #
