@@ -1,6 +1,6 @@
 """
 Wrapper classes to enhance the functionality of native pipeline classes conforming with
-the scikit-learn API.
+the `scikit-learn` API.
 """
 
 from ._wrapper import *
