@@ -1,45 +1,52 @@
 """
-Core implementation of :mod:`sklearndf.regression` loaded
-from sklearn 0.22 onwards
+Additional implementation of :mod:`sklearndf.regression` loaded
+from sklearn 1.0 onwards
 """
 import logging
-from typing import TypeVar
 
-from sklearn.base import RegressorMixin
-from sklearn.ensemble import StackingRegressor
+from sklearn.ensemble import HistGradientBoostingRegressor
+from sklearn.linear_model import QuantileRegressor
 
 from pytools.api import AllTracker
 
-from ..wrapper import make_df_regressor
-from .wrapper import StackingRegressorWrapperDF
+from ..wrapper import RegressorWrapperDF
 
 log = logging.getLogger(__name__)
 
-__all__ = ["StackingRegressorDF"]
+__all__ = ["HistGradientBoostingRegressorDF", "QuantileRegressorDF"]
 
 __imported_estimators = {name for name in globals().keys() if name.endswith("DF")}
 
-#
-# type variables
-#
-
-T_Regressor = TypeVar("T_Regressor", bound=RegressorMixin)
-
 
 #
 # Ensure all symbols introduced below are included in __all__
 #
 
-__tracker = AllTracker(globals(), allow_imported_definitions=True)
+__tracker = AllTracker(globals())
 
 
 #
-# Class definitions
+# ensemble
 #
 
-StackingRegressorDF = make_df_regressor(
-    StackingRegressor, base_wrapper=StackingRegressorWrapperDF
-)
+
+class HistGradientBoostingRegressorDF(
+    RegressorWrapperDF[HistGradientBoostingRegressor],
+    native=HistGradientBoostingRegressor,
+):
+    """Stub for DF wrapper of class ``HistGradientBoostingRegressor``"""
+
+
+#
+# linear model
+#
+
+
+class QuantileRegressorDF(
+    RegressorWrapperDF[QuantileRegressor],
+    native=QuantileRegressor,
+):
+    """Stub for DF wrapper of class ``QuantileRegressor``"""
 
 
 #
@@ -53,7 +60,6 @@ __tracker.validate()
 # validate that __all__ comprises all symbols ending in "DF", and no others
 #
 
-
 __estimators = {
     sym
     for sym in dir()
