@@ -4,22 +4,32 @@ Core implementation of :mod:`sklearndf.transformation.wrapper`
 
 import logging
 from abc import ABCMeta, abstractmethod
-from functools import reduce
-from typing import Any, Generic, List, Optional, TypeVar, Union
+from typing import (
+    Any,
+    Callable,
+    Generic,
+    Iterable,
+    List,
+    Optional,
+    Sequence,
+    TypeVar,
+    Union,
+    cast,
+)
 
 import numpy as np
+import numpy.typing as npt
 import pandas as pd
 from sklearn.base import TransformerMixin
 from sklearn.compose import ColumnTransformer
 from sklearn.impute import MissingIndicator, SimpleImputer
-from sklearn.impute._iterative import IterativeImputer
 from sklearn.kernel_approximation import AdditiveChi2Sampler
 from sklearn.manifold import Isomap
-from sklearn.preprocessing import KBinsDiscretizer, OneHotEncoder, PolynomialFeatures
+from sklearn.preprocessing import KBinsDiscretizer, OneHotEncoder
 
 from pytools.api import AllTracker
 
-from ... import TransformerDF
+from ... import TransformerDF, __sklearn_1_0__, __sklearn_version__
 from ...wrapper import TransformerWrapperDF
 
 log = logging.getLogger(__name__)
@@ -39,7 +49,7 @@ __all__ = [
     "MissingIndicatorWrapperDF",
     "AdditiveChi2SamplerWrapperDF",
     "KBinsDiscretizerWrapperDF",
-    "PolynomialFeaturesWrapperDF",
+    "PolynomialTransformerWrapperDF",
     "OneHotEncoderWrapperDF",
 ]
 
@@ -50,12 +60,17 @@ __all__ = [
 
 T_Transformer = TypeVar("T_Transformer", bound=TransformerMixin)
 
-# T_Imputer is needed because sklearn's _BaseImputer only exists from v0.22 onwards.
+# T_Imputer is needed because scikit-learn's _BaseImputer only exists from v0.22
+# onwards.
 # Once we drop support for sklearn 0.21, _BaseImputer can be used instead.
 # The following TypeVar helps to annotate availability of "add_indicator" and
 # "missing_values" attributes on an imputer instance for ImputerWrapperDF below
-T_Imputer = TypeVar("T_Imputer", SimpleImputer, IterativeImputer)
 
+# noinspection PyProtectedMember
+from sklearn.impute._iterative import IterativeImputer
+
+T_Imputer = TypeVar("T_Imputer", SimpleImputer, IterativeImputer)
+T_Polynomial = TypeVar("T_Polynomial", bound=TransformerMixin)
 
 #
 # Ensure all symbols introduced below are included in __all__
@@ -81,14 +96,16 @@ class NumpyTransformerWrapperDF(
     """
 
     # noinspection PyPep8Naming
-    def _convert_X_for_delegate(self, X: pd.DataFrame) -> Any:
-        return super()._convert_X_for_delegate(X).values
+    def _adjust_X_type_for_delegate(
+        self,
+        X: pd.DataFrame,
+    ) -> npt.NDArray[Any]:
+        return cast(npt.NDArray[Any], X.values)
 
-    def _convert_y_for_delegate(
-        self, y: Optional[Union[pd.Series, pd.DataFrame]]
-    ) -> Any:
-        y = super()._convert_y_for_delegate(y)
-        return None if y is None else y.values
+    def _adjust_y_type_for_delegate(
+        self, y: Union[pd.Series, pd.DataFrame, None]
+    ) -> Optional[npt.NDArray[Any]]:
+        return None if y is None else cast(npt.NDArray[Any], y.values)
 
 
 class ColumnSubsetTransformerWrapperDF(
@@ -142,7 +159,7 @@ class BaseMultipleInputsPerOutputTransformerWrapperDF(
         pass
 
     def _get_features_original(self) -> pd.Series:
-        raise NotImplementedError(
+        raise TypeError(
             f"{type(self.native_estimator).__name__} transformers map multiple "
             "inputs to individual output columns; current sklearndf implementation "
             "only supports many-to-1 mappings from output columns to input columns"
@@ -188,7 +205,7 @@ class NComponentsDimensionalityReductionWrapperDF(
 
     @property
     def _n_components_(self) -> int:
-        return getattr(self.native_estimator, self._ATTR_N_COMPONENTS)
+        return cast(int, getattr(self.native_estimator, self._ATTR_N_COMPONENTS))
 
 
 class ComponentsDimensionalityReductionWrapperDF(
@@ -208,7 +225,7 @@ class ComponentsDimensionalityReductionWrapperDF(
 
     # noinspection PyPep8Naming
     def _post_fit(
-        self, X: pd.DataFrame, y: Optional[pd.Series] = None, **fit_params
+        self, X: pd.DataFrame, y: Optional[pd.Series] = None, **fit_params: Any
     ) -> None:
         # noinspection PyProtectedMember
         super()._post_fit(X, y, **fit_params)
@@ -259,7 +276,10 @@ class ColumnTransformerWrapperDF(
     def _validate_delegate_estimator(self) -> None:
         column_transformer: ColumnTransformer = self.native_estimator
 
-        if column_transformer.remainder != ColumnTransformerWrapperDF.__DROP:
+        if (
+            column_transformer.remainder
+            not in ColumnTransformerWrapperDF.__SPECIAL_TRANSFORMERS
+        ):
             raise ValueError(
                 f"unsupported value for arg remainder: ({column_transformer.remainder})"
             )
@@ -291,20 +311,66 @@ class ColumnTransformerWrapperDF(
         values the corresponding input column names.
         """
 
-        return reduce(
-            lambda x, y: x.append(y),
-            (
-                (
-                    pd.Series(index=columns, data=columns)
-                    if df_transformer == ColumnTransformerWrapperDF.__PASSTHROUGH
-                    else df_transformer.feature_names_original_
+        # If True, get_feature_names_out will prefix all feature names with the name of
+        # the transformer that generated that feature
+        verbose_feature_names_out: bool = getattr(
+            self.native_estimator, "verbose_feature_names_out", False
+        )
+
+        # noinspection PyShadowingNames
+        def _features_original(
+            transformer_name: str,
+            df_transformer: Union[TransformerDF, str],
+            columns: Iterable[Any],
+        ) -> pd.Series:
+            input_column_names: npt.NDArray[Any]
+            output_column_names: npt.NDArray[Any]
+
+            if df_transformer == ColumnTransformerWrapperDF.__PASSTHROUGH:
+                # we may get positional indices for columns selected by the
+                # 'passthrough' transformer, and in that case so need to look up the
+                # associated column names
+                if all(isinstance(column, int) for column in columns):
+                    output_column_names = self._get_features_in().to_numpy()[columns]
+                else:
+                    output_column_names = np.array(columns)
+                input_column_names = output_column_names
+
+            else:
+                assert isinstance(df_transformer, TransformerDF), (
+                    "expected TransformerDF but got a "
+                    f"{type(df_transformer).__name__}: {df_transformer!r}"
+                )
+                feature_names_original_: pd.Series = (
+                    df_transformer.feature_names_original_
+                )
+                if verbose_feature_names_out:
+                    input_column_names = feature_names_original_.to_numpy()
+                    output_column_names = feature_names_original_.index.to_numpy()
+                else:
+                    return feature_names_original_
+
+            if verbose_feature_names_out:
+                output_column_names = np.array(
+                    [f"{transformer_name}__{column}" for column in output_column_names]
+                )
+
+            return pd.Series(index=output_column_names, data=input_column_names)
+
+        transformer_name: str
+        df_transformer: Union[TransformerDF, str]
+        columns: Union[Sequence[Any], npt.NDArray[Any]]
+        return pd.concat(
+            [
+                _features_original(transformer_name, df_transformer, columns)
+                for transformer_name, df_transformer, columns in (
+                    self.native_estimator.transformers_
                 )
-                for _, df_transformer, columns in self.native_estimator.transformers_
                 if (
                     len(columns) > 0
                     and df_transformer != ColumnTransformerWrapperDF.__DROP
                 )
-            ),
+            ]
         )
 
 
@@ -317,18 +383,19 @@ class ImputerWrapperDF(TransformerWrapperDF[T_Imputer], metaclass=ABCMeta):
         # get the columns that were dropped during imputation
         delegate_estimator = self.native_estimator
 
-        nan_mask = []
+        nan_mask: Union[List[bool], npt.NDArray[Any]] = []
 
-        def _nan_mask_from_statistics(stats: np.array):
+        def _nan_mask_from_statistics(
+            stats: npt.NDArray[Any],
+        ) -> Union[List[bool], npt.NDArray[np.bool_]]:
             if issubclass(stats.dtype.type, float):
-                na_mask = np.isnan(stats)
+                return cast(npt.NDArray[np.bool_], np.isnan(stats))
             else:
-                na_mask = [
+                return [
                     x is None or (isinstance(x, float) and np.isnan(x)) for x in stats
                 ]
-            return na_mask
 
-        # implementation for i.e. SimpleImputer
+        # implementation for SimpleImputer
         if hasattr(delegate_estimator, "statistics_"):
             nan_mask = _nan_mask_from_statistics(stats=delegate_estimator.statistics_)
 
@@ -337,13 +404,13 @@ class ImputerWrapperDF(TransformerWrapperDF[T_Imputer], metaclass=ABCMeta):
             initial_imputer: SimpleImputer = delegate_estimator.initial_imputer_
             nan_mask = _nan_mask_from_statistics(stats=initial_imputer.statistics_)
 
-        # implementation for i.e. KNNImputer
+        # implementation for KNNImputer
         elif hasattr(delegate_estimator, "_mask_fit_X"):
             # noinspection PyProtectedMember
             nan_mask = np.all(delegate_estimator._mask_fit_X, axis=0)
 
         # the imputed columns are all ingoing columns, except the ones that were dropped
-        imputed_columns = self.feature_names_in_.delete(np.argwhere(nan_mask))
+        imputed_columns = self.feature_names_in_.delete(np.argwhere(nan_mask).tolist())
         features_original = pd.Series(
             index=imputed_columns, data=imputed_columns.values
         )
@@ -357,7 +424,9 @@ class ImputerWrapperDF(TransformerWrapperDF[T_Imputer], metaclass=ABCMeta):
                 features_in=self.feature_names_in_,
                 n_outputs=self.n_outputs_,
             )
-            return features_original.append(missing_indicator.feature_names_original_)
+            return pd.concat(
+                [features_original, missing_indicator.feature_names_original_]
+            )
         else:
             return features_original
 
@@ -370,7 +439,7 @@ class MissingIndicatorWrapperDF(
     """
 
     def _get_features_original(self) -> pd.Series:
-        features_original: np.ndarray = self.feature_names_in_[
+        features_original: npt.NDArray[Any] = self.feature_names_in_[
             self.native_estimator.features_
         ].values
         features_out = pd.Index([f"{name}__missing" for name in features_original])
@@ -384,7 +453,7 @@ class IsomapWrapperDF(BaseDimensionalityReductionWrapperDF[Isomap], metaclass=AB
 
     @property
     def _n_components_(self) -> int:
-        return self.native_estimator.embedding_.shape[1]
+        return cast(int, self.native_estimator.embedding_.shape[1])
 
 
 class AdditiveChi2SamplerWrapperDF(
@@ -396,22 +465,26 @@ class AdditiveChi2SamplerWrapperDF(
 
     @property
     def _n_components_(self) -> int:
-        return len(self._features_in) * (2 * self.native_estimator.sample_steps + 1)
+        assert self._features_in is not None, "estimator is fitted"
+        return len(self._features_in) * (
+            2 * cast(int, self.native_estimator.sample_steps) + 1
+        )
 
 
-class PolynomialFeaturesWrapperDF(
-    BaseMultipleInputsPerOutputTransformerWrapperDF[PolynomialFeatures],
+class PolynomialTransformerWrapperDF(
+    BaseMultipleInputsPerOutputTransformerWrapperDF[T_Polynomial],
+    Generic[T_Polynomial],
     metaclass=ABCMeta,
 ):
     """
-    DF wrapper for :class:`sklearn.preprocessing.PolynomialFeatures`.
+    DF wrapper for :class:`sklearn.preprocessing.PolynomialFeatures`
+    and :class:`sklearn.preprocessing.SplineTransformer`.
     """
 
     def _get_features_out(self) -> pd.Index:
-        return pd.Index(
-            data=self.native_estimator.get_feature_names(
-                input_features=self.feature_names_in_.astype(str)
-            )
+        return _get_native_feature_names_out(
+            feature_names_in_=self.feature_names_in_,
+            native_estimator=self.native_estimator,
         )
 
 
@@ -430,36 +503,39 @@ class OneHotEncoderWrapperDF(TransformerWrapperDF[OneHotEncoder], metaclass=ABCM
         # Remove 1st category column if argument drop == 'first'
         # Remove 1st category column only of binary features if arg drop == 'if_binary'
 
-        feature_names_out = pd.Index(
-            self.native_estimator.get_feature_names(self.feature_names_in_)
+        native_estimator: OneHotEncoder = self.native_estimator
+        feature_names_in: pd.Index = self.feature_names_in_
+
+        feature_names_out = _get_native_feature_names_out(
+            feature_names_in_=feature_names_in, native_estimator=native_estimator
         )
 
         if self.drop == "first":
-            feature_names_in = [
+            feature_names_in_mapped = [
                 column_original
                 for column_original, category in zip(
-                    self.feature_names_in_, self.native_estimator.categories_
+                    feature_names_in, native_estimator.categories_
                 )
                 for _ in range(len(category) - 1)
             ]
         elif self.drop == "if_binary":
-            feature_names_in = [
+            feature_names_in_mapped = [
                 column_original
                 for column_original, category in zip(
-                    self.feature_names_in_, self.native_estimator.categories_
+                    feature_names_in, native_estimator.categories_
                 )
                 for _ in (range(1) if len(category) == 2 else category)
             ]
         else:
-            feature_names_in = [
+            feature_names_in_mapped = [
                 column_original
                 for column_original, category in zip(
-                    self.feature_names_in_, self.native_estimator.categories_
+                    feature_names_in, native_estimator.categories_
                 )
                 for _ in category
             ]
 
-        return pd.Series(index=feature_names_out, data=feature_names_in)
+        return pd.Series(index=feature_names_out, data=feature_names_in_mapped)
 
 
 class KBinsDiscretizerWrapperDF(
@@ -508,6 +584,28 @@ class KBinsDiscretizerWrapperDF(
             )
 
 
+#
+# Helper functions
+#
+
+
+def _get_native_feature_names_out(
+    feature_names_in_: pd.Index, native_estimator: TransformerMixin
+) -> pd.Index:
+    # get the output feature names from a native transformer implementing
+    # method get_feature_names() (sklearn 0.x) or get_feature_names_out() (sklearn 1.x)
+
+    get_feature_names_out_fn: Callable[[npt.NDArray[Any]], npt.NDArray[Any]]
+    if __sklearn_version__ >= __sklearn_1_0__:
+        # noinspection PyUnresolvedReferences
+        get_feature_names_out_fn = native_estimator.get_feature_names_out
+    else:
+        # noinspection PyUnresolvedReferences
+        get_feature_names_out_fn = native_estimator.get_feature_names
+
+    return pd.Index(get_feature_names_out_fn(feature_names_in_.to_numpy().astype(str)))
+
+
 #
 # validate __all__
 #
