@@ -7,8 +7,10 @@ from packaging import version
 from test.paths import TEST_CONFIG_YML
 
 
-def check_sklearn_version(minimum: Optional[str], maximum: Optional[str] = None):
-    """ Utility to check sklearn version against provided string. """
+def check_sklearn_version(
+    minimum: Optional[str], maximum: Optional[str] = None
+) -> bool:
+    """Utility to check sklearn version against provided string."""
     v_sklearn = version.parse(sklearn.__version__)
     return not (
         (minimum and version.parse(minimum) > v_sklearn)
