@@ -1,5 +1,6 @@
 # inspired by:
 # https://github.com/scikit-learn/scikit-learn/blob/master/sklearn/tests/test_base.py
+from typing import Any
 
 import numpy as np
 import scipy.sparse as sp
@@ -9,34 +10,49 @@ from sklearn.base import BaseEstimator, is_classifier
 from sklearn.model_selection import GridSearchCV
 from sklearn.pipeline import Pipeline
 
-import sklearndf
+from pytools.expression import freeze, make_expression
+from pytools.expression.atomic import Id
+
 from sklearndf.classification import SVCDF, DecisionTreeClassifierDF
 from sklearndf.pipeline import PipelineDF
 from sklearndf.transformation import OneHotEncoderDF
-from sklearndf.wrapper import make_df_estimator
+from sklearndf.wrapper import EstimatorWrapperDF
 
 
-class _DummyEstimator(BaseEstimator):
-    def __init__(self, l1=0, empty=None) -> None:
+class DummyEstimator(
+    BaseEstimator,  # type: ignore
+):
+    def __init__(self, l1: int = 0, empty: Any = None) -> None:
         self.l1 = l1
         self.empty = empty
 
 
-class _DummyEstimator2(BaseEstimator):
-    def __init__(self, a=None, b=None) -> None:
+class DummyEstimator2(
+    BaseEstimator,  # type: ignore
+):
+    def __init__(self, a: Any = None, b: Any = None) -> None:
         self.a = a
         self.b = b
 
 
-class _DummyEstimator3(BaseEstimator):
-    def __init__(self, c=0, d=None) -> None:
+class DummyEstimator3(
+    BaseEstimator,  # type: ignore
+):
+    def __init__(self, c: int = 0, d: Any = None) -> None:
         self.c = c
         self.d = d
 
 
-_DummyEstimatorDF = make_df_estimator(_DummyEstimator)
-_DummyEstimator2DF = make_df_estimator(_DummyEstimator2)
-_DummyEstimator3DF = make_df_estimator(_DummyEstimator3)
+class DummyEstimatorDF(EstimatorWrapperDF[DummyEstimator], native=DummyEstimator):
+    """A trivial estimator."""
+
+
+class DummyEstimator2DF(EstimatorWrapperDF[DummyEstimator2], native=DummyEstimator2):
+    """A trivial estimator."""
+
+
+class DummyEstimator3DF(EstimatorWrapperDF[DummyEstimator3], native=DummyEstimator3):
+    """A trivial estimator."""
 
 
 def test_clone() -> None:
@@ -46,7 +62,7 @@ def test_clone() -> None:
     # and check that the obtained copy is a correct deep copy.
 
     encoder = OneHotEncoderDF(drop="first", sparse=False)
-    new_encoder = clone(encoder)
+    new_encoder = encoder.clone()
     assert encoder is not new_encoder
     assert encoder.get_params() == new_encoder.get_params()
 
@@ -65,26 +81,26 @@ def test_clone_2() -> None:
     encoder = OneHotEncoderDF(drop="first", sparse=False)
 
     encoder.own_attribute = "test"
-    new_encoder = clone(encoder)
+    new_encoder = encoder.clone()
 
     assert not hasattr(new_encoder, "own_attribute")
 
 
 def test_clone_empty_array() -> None:
     # Regression test for cloning estimators with empty arrays
-    clf = _DummyEstimatorDF(empty=np.array([]))
-    clf2 = clone(clf)
+    clf = DummyEstimatorDF(empty=np.array([]))
+    clf2 = clf.clone()
     assert_array_equal(clf.empty, clf2.empty)
 
-    clf = _DummyEstimatorDF(empty=sp.csr_matrix(np.array([[0]])))
-    clf2 = clone(clf)
+    clf = DummyEstimatorDF(empty=sp.csr_matrix(np.array([[0]])))
+    clf2 = clf.clone()
     assert_array_equal(clf.empty.data, clf2.empty.data)
 
 
 def test_clone_nan() -> None:
     # Regression test for cloning estimators with default parameter as np.nan
-    clf = _DummyEstimatorDF(empty=np.nan)
-    clf2 = clone(clf)
+    clf = DummyEstimatorDF(empty=np.nan)
+    clf2 = clf.clone()
 
     assert clf.empty is clf2.empty
 
@@ -96,8 +112,8 @@ def test_clone_sparse_matrices() -> None:
 
     for cls in sparse_matrix_classes:
         sparse_matrix = cls(np.eye(5))
-        clf = _DummyEstimatorDF(empty=sparse_matrix)
-        clf_cloned = clone(clf)
+        clf = DummyEstimatorDF(empty=sparse_matrix)
+        clf_cloned = clf.clone()
         assert clf.empty.__class__ is clf_cloned.empty.__class__
         assert_array_equal(clf.empty.toarray(), clf_cloned.empty.toarray())
 
@@ -105,47 +121,40 @@ def test_clone_sparse_matrices() -> None:
 def test_clone_estimator_types() -> None:
     # Check that clone works for parameters that are types rather than
     # instances
-    clf = _DummyEstimatorDF(empty=_DummyEstimator)
-    clf2 = clone(clf)
+    clf = DummyEstimatorDF(empty=DummyEstimator)
+    clf2 = clf.clone()
 
     assert clf.empty is clf2.empty
 
 
 def test_repr() -> None:
     # Smoke test the repr of the base estimator.
-    my_estimator = _DummyEstimatorDF()
-    repr(my_estimator)
+    repr(DummyEstimatorDF())
 
-    if sklearndf.__sklearn_version__ < sklearndf.__sklearn_0_23__:
-        expected_repr = (
-            "_DummyEstimator2DF(a=_DummyEstimator3DF(c=None, d=None),\n"
-            "                   b=_DummyEstimator3DF(c=1, d=2))"
+    estimator = DummyEstimator2DF(
+        a=DummyEstimator3DF(c=None), b=DummyEstimator3DF(c=1, d=2)
+    )
+    assert freeze(make_expression(estimator)) == freeze(
+        Id.DummyEstimator2DF(
+            a=Id.DummyEstimator3DF(c=None), b=Id.DummyEstimator3DF(c=1, d=2)
         )
-        expected_len = 702
-    else:
-        expected_repr = (
-            "_DummyEstimator2DF(a=_DummyEstimator3DF(c=None), "
-            "b=_DummyEstimator3DF(c=1, d=2))"
-        )
-        expected_len = 675
-
-    estimator_1 = _DummyEstimator2DF(
-        _DummyEstimator3DF(c=None), _DummyEstimator3DF(c=1, d=2)
+    )
+    assert repr(estimator) == (
+        "DummyEstimator2DF(a=DummyEstimator3DF(c=None), "
+        "b=DummyEstimator3DF(c=1, d=2))"
     )
-    assert repr(estimator_1) == expected_repr
 
-    estimator_2 = _DummyEstimator2DF(a=["long_params"] * 1000)
-    assert len(repr(estimator_2)) == expected_len
+    assert len(repr(DummyEstimator2DF(a=["long_params"] * 1000))) == 15021
 
 
 def test_str() -> None:
     # Smoke test the str of the base estimator
-    my_estimator = _DummyEstimatorDF()
+    my_estimator = DummyEstimatorDF()
     str(my_estimator)
 
 
 def test_get_params() -> None:
-    test = _DummyEstimator2DF(_DummyEstimator3DF(), _DummyEstimator3DF())
+    test = DummyEstimator2DF(DummyEstimator3DF(), DummyEstimator3DF())
 
     assert "a__d" in test.get_params(deep=True)
     assert "a__d" not in test.get_params(deep=False)
