@@ -1,4 +1,4 @@
-from typing import List, Type, cast
+from typing import Any, List, Type, cast
 
 import numpy as np
 import pandas as pd
@@ -22,15 +22,17 @@ from sklearndf import (
     ClassifierDF,
     RegressorDF,
     TransformerDF,
+    __sklearn_0_22__,
     __sklearn_0_24__,
+    __sklearn_1_0__,
     __sklearn_version__,
 )
 from sklearndf.classification import RandomForestClassifierDF
+from sklearndf.clustering import FeatureAgglomerationDF
 from sklearndf.transformation import (
     RFECVDF,
     RFEDF,
     ColumnTransformerDF,
-    FeatureAgglomerationDF,
     KBinsDiscretizerDF,
     NormalizerDF,
     OneHotEncoderDF,
@@ -39,7 +41,6 @@ from sklearndf.transformation import (
     SparseCoderDF,
     StandardScalerDF,
 )
-from sklearndf.transformation.extra import OutlierRemoverDF
 from sklearndf.wrapper import TransformerWrapperDF
 
 TRANSFORMER_EXCLUSIONS = [
@@ -59,14 +60,30 @@ if check_sklearn_version(minimum="0.24"):
 
     TRANSFORMER_EXCLUSIONS.append(SequentialFeatureSelectorDF.__name__)
 
+# noinspection PyTypeChecker
 TRANSFORMERS_TO_TEST = iterate_classes(
     from_modules=sklearndf.transformation,
     matching=r".*DF",
     excluding=TRANSFORMER_EXCLUSIONS,
 )
+TRANSFORMERS_TO_TEST.append(FeatureAgglomerationDF)
 
 
-@pytest.fixture
+def test_transformer_count() -> None:
+    n = len(TRANSFORMERS_TO_TEST)
+
+    print(f"Testing {n} transformers.")
+    if __sklearn_version__ < __sklearn_0_22__:
+        assert n == 53
+    elif __sklearn_version__ < __sklearn_0_24__:
+        assert n == 54
+    elif __sklearn_version__ < __sklearn_1_0__:
+        assert n == 55
+    else:
+        assert n == 56
+
+
+@pytest.fixture  # type: ignore
 def test_data() -> pd.DataFrame:
     return pd.DataFrame(
         data={
@@ -78,7 +95,9 @@ def test_data() -> pd.DataFrame:
     )
 
 
-@pytest.mark.parametrize(argnames="sklearndf_cls", argvalues=TRANSFORMERS_TO_TEST)
+@pytest.mark.parametrize(  # type: ignore
+    argnames="sklearndf_cls", argvalues=TRANSFORMERS_TO_TEST
+)
 def test_wrapped_constructor(sklearndf_cls: Type[TransformerDF]) -> None:
     transformer_df: TransformerDF = sklearndf_cls()
 
@@ -125,7 +144,7 @@ def test_special_wrapped_constructors() -> None:
         SequentialFeatureSelectorDF(estimator=rf)
 
 
-@pytest.mark.parametrize(
+@pytest.mark.parametrize(  # type: ignore
     argnames="sklearn_cls",
     argvalues=iterate_classes(
         from_modules=sklearn.preprocessing,
@@ -139,6 +158,7 @@ def test_fit_transform(
     test_data = test_data.select_dtypes(include=float)
 
     # get the wrapped counterpart for sklearn:
+    # noinspection PyTypeChecker
     wrapper_class = get_sklearndf_wrapper_class(
         to_wrap=sklearn_cls, from_module=sklearndf.transformation
     )
@@ -147,7 +167,7 @@ def test_fit_transform(
 
     # initialize both kind of transformers
     transformer_native = cast(TransformerMixin, sklearn_cls())
-    transformer_df = cast(TransformerDF, wrapper_class())
+    transformer_df = wrapper_class()
 
     # for sklearn >=0.22 - check if not_fitted error is raised properly:
     check_expected_not_fitted_error(estimator=transformer_df)
@@ -176,6 +196,24 @@ def test_fit_transform(
         inverse_transformed_df, test_data.rename_axis(columns="feature_in")
     )
 
+    # test feature names in and out
+    if __sklearn_version__ >= __sklearn_1_0__:
+        # noinspection PyUnresolvedReferences
+        assert_array_equal(
+            transformer_df.feature_names_in_.values,
+            transformer_native.feature_names_in_,
+        )
+        # noinspection PyUnresolvedReferences
+        assert_array_equal(
+            transformer_df.feature_names_out_.values,
+            transformer_native.get_feature_names_out(),
+        )
+        # noinspection PyUnresolvedReferences
+        assert_array_equal(
+            transformer_df.feature_names_original_.index.values,
+            transformer_native.get_feature_names_out(),
+        )
+
 
 def test_column_transformer(test_data: pd.DataFrame) -> None:
     numeric_columns: List[str] = test_data.select_dtypes(include=float).columns.tolist()
@@ -183,11 +221,14 @@ def test_column_transformer(test_data: pd.DataFrame) -> None:
 
     feature_names_in_expected = test_data.columns.rename("feature_in")
 
-    for remainder, output_names in [
-        ("drop", ["c0", "c2", "c3"]),
-        ("passthrough", ["c0", "c2", "c3", "c1"]),
-    ]:
-        feature_names_out_expected = pd.Index(output_names, name="feature_out")
+    # noinspection PyShadowingNames
+    def _test_transformer(
+        remainder: str,
+        names_in: List[str],
+        names_out: List[str],
+        **transformer_args: Any,
+    ) -> None:
+        feature_names_out_expected = pd.Index(names_out, name="feature_out")
 
         # test fit-transform in connection with ColumnTransformer(DF)
         tx_df = StandardScalerDF()
@@ -197,6 +238,7 @@ def test_column_transformer(test_data: pd.DataFrame) -> None:
                 ("keep", "passthrough", ["c3"]),
             ],
             remainder=remainder,
+            **transformer_args,
         )
         transformed_df = col_tx_df.fit_transform(X=test_data)
 
@@ -207,6 +249,7 @@ def test_column_transformer(test_data: pd.DataFrame) -> None:
                 ("keep", "passthrough", ["c3"]),
             ],
             remainder=remainder,
+            **transformer_args,
         )
         transformed_native = col_tx_native.fit_transform(X=test_data)
 
@@ -218,9 +261,49 @@ def test_column_transformer(test_data: pd.DataFrame) -> None:
         assert col_tx_df.feature_names_in_.equals(feature_names_in_expected)
         assert col_tx_df.feature_names_out_.equals(feature_names_out_expected)
         assert col_tx_df.feature_names_original_.equals(
-            pd.Series(feature_names_out_expected, index=feature_names_out_expected)
+            pd.Series(names_in, index=feature_names_out_expected)
         )
 
+        if __sklearn_version__ >= __sklearn_1_0__:
+            # noinspection PyUnresolvedReferences
+            assert_array_equal(
+                col_tx_df.feature_names_in_.values, col_tx_native.feature_names_in_
+            )
+
+    for remainder, names, names_verbose in [
+        (
+            "drop",
+            ["c0", "c2", "c3"],
+            ["tx__c0", "tx__c2", "keep__c3"],
+        ),
+        (
+            "passthrough",
+            ["c0", "c2", "c3", "c1"],
+            ["tx__c0", "tx__c2", "keep__c3", "remainder__c1"],
+        ),
+    ]:
+        if __sklearn_version__ < __sklearn_1_0__:
+            _test_transformer(remainder=remainder, names_in=names, names_out=names)
+        else:
+            # As of scikit-learn 1.0, column transformers have a new boolean parameter
+            # verbose_feature_names_out. We test once with verbosity disabled, once
+            # with verbosity enabled, and once with the default (enabled).
+            _test_transformer(
+                remainder=remainder,
+                names_in=names,
+                names_out=names,
+                verbose_feature_names_out=False,
+            )
+            _test_transformer(
+                remainder=remainder,
+                names_in=names,
+                names_out=names_verbose,
+                verbose_feature_names_out=True,
+            )
+            _test_transformer(
+                remainder=remainder, names_in=names, names_out=names_verbose
+            )
+
 
 def test_normalizer_df() -> None:
     x = [[4.0, 1.0, 2.0, 2.0], [1.0, 3.0, 9.0, 3.0], [5.0, 7.0, 5.0, 1.0]]
@@ -262,7 +345,7 @@ def test_simple_imputer_df() -> None:
     imputer_native = SimpleImputer(add_indicator=True)
     imputer_df = SimpleImputerDF(add_indicator=True)
 
-    transformed_native = imputer_native.fit_transform(X=x)
+    transformed_native = imputer_native.fit_transform(X=x_df)
     transformed_df_expected = pd.DataFrame(
         transformed_native,
         columns=pd.Index(
@@ -288,8 +371,15 @@ def test_simple_imputer_df() -> None:
         inverse_transformed_df = imputer_df.inverse_transform(X=transformed_df)
         assert_frame_equal(inverse_transformed_df, x_df)
 
+    # test feature name in
+    if __sklearn_version__ >= __sklearn_1_0__:
+        # noinspection PyUnresolvedReferences
+        assert_array_equal(
+            imputer_df.feature_names_in_.values, imputer_native.feature_names_in_
+        )
 
-@pytest.fixture
+
+@pytest.fixture  # type: ignore
 def df_outlier() -> pd.DataFrame:
     return pd.DataFrame(
         data={
@@ -301,20 +391,6 @@ def df_outlier() -> pd.DataFrame:
     )
 
 
-def test_outlier_remover(df_outlier: pd.DataFrame) -> None:
-    outlier_remover = OutlierRemoverDF(iqr_multiple=2)
-    df_transformed = outlier_remover.fit_transform(df_outlier)
-    df_transformed_expected = pd.DataFrame(
-        data={
-            "c0": [0, 1, 2, 3, 4],
-            "c1": [np.nan, 0, 0, 0, np.nan],
-            "c2": [np.nan, 0, 1, 2, 3],
-            "c3": [0, 1, 2, 3, np.nan],
-        }
-    )
-    assert_frame_equal(df_transformed, df_transformed_expected)
-
-
 def test_one_hot_encoding() -> None:
     test_data_categorical = pd.DataFrame(
         data=[
